<?php

class Xoxo_Object_Exportable extends Xoxo_Object
{
  /**
   * @var string
   */
  protected $_name;

  /**
   * Set machine name.
   * 
   * @param string $name
   *   Machine name.
   * 
   * @return Xoxo_Object_Exportable
   *   Self reference for chaining.
   */
  public function setName($name) {
    $this->_name = $name;
    return $this;
  }

  /**
   * Get machine name.
   * 
   * @return string
   * 
   * @throws Oox_Exception
   *   If name not set.
   */
  public function getName() {
    if (!$this->_name) {
      throw new Oox_Exception("Name not set.");
    }
    return $this->_name;
  }

  /**
   * Render the preview page.
   */
  public function renderPreview() { return $this->getName(); }

  /**
   * Default constructor.
   * 
   * @param string $name = NULL
   *   (optional) Machine name. Use this parameter if you intend to save or
   *   export your layout later.
   * @param string $description = NULL
   *   (optional) Human readable description.
   */
  public function __construct($name = NULL, $description = NULL) {
    parent::__construct($description);
    if ($name) {
      $this->_name = $name;
    }
  }

  /**
   * @see Xoxo_Object::duplicate()
   */
  public function duplicate() {
    $object = parent::duplicate();
    $object->setName(NULL);
    return $object;
  }

  /**
   * @see Xoxo_Object::getSchema()
   */
  public function getSchema() {
    $config = parent::getSchema();
    $config->set('name', $this->_name);
    return $config;
  }

  /**
   * @see Formable_Interface::getForm()
   */
  public function getForm($formIdentifier = Formable_Interface::FORM_DEFAULT) {
    if (Formable_Interface::FORM_DEFAULT == $formIdentifier) {
      return new Xoxo_Object_ExportableFormDefault;
    }
  }
}
