<?php

class Xoxo_Object_ExportableFormDefault extends Xoxo_ObjectFormDefault
{
  /**
   * @see Oox_Formable_Interface::form()
   */
  public function formBuild(array &$form, array $values = array(), $parent = NULL) {
    parent::formBuild($form, $values, $parent);
    // Reorder elements.
    $form['xoxo']['description']['#weight'] = -1;
    $form['xoxo']['name'] = array(
      '#type' => 'machine_name',
      '#maxlength' => 128,
      // Source is problematic here because we can't guess where we are in the form.
      '#machine_name' => array(
        'source' => array_merge($this->getParents(), array('xoxo', 'description')),
        'exists' => 'xoxo_return_false',
      ),
      '#description' => t('A unique machine-readable type for this object.'),
      '#weight' => -2,
    );
    // Disable this field for already saved objects. Name is another primary
    // key for exportables.
    if ($parent->isStored()) {
      $form['xoxo']['name']['#disabled'] = TRUE;
      $form['xoxo']['name']['#value'] = $parent->getName();
    }
  }

  /**
   * @see Oox_Formable_Interface::formValidate()
   */
  public function formValidate(array &$values, $parent = NULL) {
    if (!$errors = parent::formSubmit($values, $parent)) {
      $errors = array();
    }
    // Check the name matches the right formatting rules, only in case our
    // object is a new instance being created. Once the object is created no
    // name override will ever be possible.
    if ($parent->isStored()) {
      if (!preg_match('/^[a-zA-Z0-9_-]+$/', trim($values['xoxo']['name']))) {
        $errors['xoxo']['name'] = t("Title contains illegal characters.");
      }
    }
    return $errors;
  }

  /**
   * @see Oox_Formable_Interface::formSubmit()
   */
  public function formSubmit(array &$values, $parent = NULL) {
    parent::formSubmit($values, $parent);
    if (!$parent->isStored()) {
      $parent->setName($values['xoxo']['name']);
    }
  }
}
