<?php
// $Id$

/**
 * Xoxo Entity Proxy default controller.
 */
class XoxoEntity_Controller
  implements DrupalEntityControllerInterface
{
  /**
   * Current entity type. This is the storage internal type, we keep here
   * for direct access without using the Xoxo_Storage::getType() accessor
   * anymore.
   * 
   * @var $entityType
   */
  protected $_entityType;

  /**
   * Current storage.
   * 
   * @var Xoxo_Storage
   */
  protected $_storage;

  /**
   * @see DrupalEntityControllerInterface::__construct()
   */
  public function __construct($entityType) {
    $this->_entityType = $entityType;
    $this->_storage = Oox_Registry::getInstance('storage')->getItem($entityType);
  }

  /**
   * @see DrupalEntityControllerInterface::resetCache()
   */
  public function resetCache(array $ids = NULL) {
    $this->_storage->cacheClear(isset($ids) ? $ids : NULL);
  }

  /**
   * Copy of actual controller method from core.
   * 
   * @see DrupalDefaultEntityController::attachLoad()
   */
  protected function attachLoad($objects, $revision_id = FALSE) {
    // Attach fields.
    field_attach_load($this->_entityType, $objects);

    // Call hook_entity_load().
    foreach (module_implements('entity_load') as $module) {
      $function = $module . '_entity_load';
      $function($objects, $this->_entityType);
    }
    // Call hook_TYPE_load(). The first argument for hook_TYPE_load() are
    // always the queried entities, followed by additional arguments set in
    // $this->hookLoadArguments.
    $args = array($objects);
    $hook = 'entity_' . $this->_entityType . '_load';
    foreach (module_implements($hook) as $module) {
      call_user_func_array($module . '_' . $hook, $args);
    }
  }

  /**
   * @see DrupalEntityControllerInterface::load()
   */
  public function load($ids = array(), $conditions = array()) {
    $entities = array();

    // Load any remaining entities from the database. This is the case if $ids
    // is set to FALSE (so we load all entities), if there are any ids left to
    // load, if loading a revision, or if $conditions was passed without $ids.
    if ($conditions) {
      // Build the query, we will always do it if we have conditions.
      $query = db_select($this->_storage->getTableName(), 'base');
      $query->addTag($this->_entityType . '_load_multiple');
      $query->fields('base', array('oid'));
      if ($ids) {
        $query->condition('base.oid', $ids, 'IN');
      }
      foreach ($conditions as $field => $value) {
        $query->condition('base.' . $field, $value);
      }
      // Execute the query, and truncate the current ids array with new result
      // in case we have conditions and want only a subset.
      $ids = $query->execute()->fetchCol();
    }

    $objects = $this->_storage->load($ids);

    $attach_objects = array();
    foreach ($objects as $oid => $object) {
      // Mark non already loaded objects as being processed for field attach.
      if (!isset($object->attached)) {
        $object->attached = TRUE;
        $attach_objects[$oid] = $object;
      }
      // Add some basic data to all objects, that will be reused by the field
      // API.
      $object->oid = $oid;
      $object->type = $object->getType();
    }

    // Pass all entities loaded from the database through $this->attachLoad(),
    // which attaches fields (if supported by the entity type) and calls the
    // entity type specific load callback, for example hook_node_load().
    if (!empty($attach_objects)) {
      $this->attachLoad($attach_objects);
    }

    return $objects;
  }
}
