<?php
// $Id$

/**
 * @file
 * Xoxo Entity Proxy administration screen.
 */

/**
 * Global overview form, can display either a named objects listing, or an
 * integer identified objects listing.
 */
function xoxo_entity_overview($form, &$form_state) {
  $form['#theme'] = 'xoxo_entity_overview';

  $customInfo = variable_get('xoxo_entities', array());

  foreach (xoxo_entity_entity_info() as $entityType => $info) {
    $storage = xoxo_storage_get($entityType);
    $registry = $storage->getItemRegistry();
    $desc = $storage->getRegistry()->getItemCache($entityType);
    $custom = isset($customInfo[$entityType]);

    $form['entities'][$entityType]['name'] = array(
      '#prefix' => '<strong>',
      '#suffix' => '</strong>',
      '#markup' => isset($desc['name']) ? t($desc['name']) : $entityType,
    );
    $form['entities'][$entityType]['type'] = array('#markup' => $entityType);
    $form['entities'][$entityType]['description'] = array(
      '#prefix' => '<small>',
      '#suffix' => '</small>',
      '#markup' => isset($desc['description']) ? t($desc['description']) : '<em>' . t("None") . '</em>',
    );
    $form['entities'][$entityType]['item_count'] = array('#markup' => '<strong>' . count($registry->getItemCache()) . '</strong>');
    $modules = system_list('module_enabled');
    $form['entities'][$entityType]['module'] = array('#markup' => isset($modules[$desc['module']]) ? t($modules[$desc['module']]->info['name']) : '<em>' . t("Unknown") . '</em>');

    $form['entities'][$entityType]['custom'] = array('#markup' => $custom ? t("Yes") : t("No"));

    // Operations.
    $links = array();
    if ($custom) {
      $links[] = array(
        'href' => 'admin/objects/registry/' . $registry->getType(),
        'title' => t('Bundles'),
      );
      $links[] = array(
        'href' => 'admin/objects/registry/' . $registry->getType() . '/edit',
        'title' => t('Edit'),
      );
    }
    else {
      $links[] = array(
        'href' => 'admin/objects/registry/' . $registry->getType(),
        'title' => t('Items'),
      );
    }
    $form['entities'][$entityType]['operations'] = array('#markup' => theme('links', array('links' => $links)));
  }

  return $form;
}

/**
 * Tells if the current entity type already exists.
 */
function xoxo_entity_exists($entityType) {
  $info = entity_get_info($entityType);
  return !empty($info);
}

/**
 * Add custom entity form.
 */
function xoxo_entity_custom_edit_form($form, &$form_state, Oox_Registry $registry = NULL) {
  if (isset($registry)) {
    $info = xoxo_entity_xoxo_storage();
    $registryType = $registry->getType();
    if (isset($info[$registryType])) {
      $storageInfo = $info[$registryType];
    }
  }
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => 'Name',
    '#default_value' => isset($storageInfo) ? $storageInfo['name'] : NULL,
    '#description' => t("Administrative entity english name."), 
  );
  $form['entityType'] = array(
    '#type' => 'machine_name',
    '#maxlength' => 32,
    '#disabled' => isset($storageInfo),
    '#machine_name' => array('exists' => 'xoxo_entity_exists'),
    '#description' => t('A unique machine-readable type for this entity.'),
    '#default_value' => isset($storageInfo) ? $registryType : NULL,
  );
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t("Administrative description"),
    '#rows' => 2,
    '#description' => t("Object english extended description for administration screens and help. This must be plain text."),
    '#default_value' => isset($storageInfo) ? $storageInfo['description'] : NULL,
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => isset($storageInfo) ? t("Update") : t("Create"),
    '#submit' => array('xoxo_entity_custom_edit_form_submit'),
  );
  if (isset($storageInfo)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t("Delete"),
      '#submit' => array('xoxo_entity_custom_edit_form_submit_delete')
    );
  }
  return $form;
}

/**
 * Submit entity.
 */
function xoxo_entity_custom_edit_form_submit($form, &$form_state) {
  require_once dirname(__FILE__) . '/xoxo_entity.helpers.inc';

  // FIXME: Change this ugly variable.
  $entities = variable_get('xoxo_entities', array());
  $entityType = $form_state['values']['entityType'];

  $entities[$entityType] = array(
    'name' => $form_state['values']['name'],
    'class' => 'Xoxo_Storage',
    'description' => $form_state['values']['description'],
  );

  // Create table.
  $table = 'xoxo_entity_' . $entityType;
  db_create_table($table, xoxo_entity_api_table_schema());

  // Set the variable only once we created the table.
  variable_set('xoxo_entities', $entities);
  drupal_flush_all_caches();

  $form_state['redirect'] = 'admin/objects/entities';
  drupal_set_message(t("New entity type <em>@entity-type</em> has been created.", array('@entity-type' => $form_state['values']['name'])));
}

/**
 * Delete entity.
 */
function xoxo_entity_custom_edit_form_submit_delete($form, &$form_state) {
  $entities = variable_get('xoxo_entities', array());
  $entityType = $form_state['values']['entityType'];
  $name = $entities[$entityType]['name'];

  // Remove current information from our variable.
  unset($entities[$entityType]);

  // Drop table.
  $table = 'xoxo_entity_' . $entityType;
  db_drop_table($table);

  // Save the variable back, without this entity.
  variable_set('xoxo_entities', $entities);
  drupal_flush_all_caches();

  $form_state['redirect'] = 'admin/objects/entities';
  drupal_set_message(t("Entity type <em>@entity-type</em> has been dropped and data removed.", array('@entity-type' => $name)));
}

/**
 * Add custom bundle (regitry item) for custom entity (storage).
 */
function xoxo_entity_custom_edit_bundle_form($form, &$form_state, $registry, $itemInfo = NULL) {
  $form['#registry_type'] = $registry->getType();
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => 'Name',
    '#default_value' => $itemInfo ? $itemInfo['name'] : NULL,
    '#description' => t("Administrative entity english name."), 
  );
  $form['entityType'] = array(
    '#type' => 'machine_name',
//    '#default_value' => $type->type,
    '#maxlength' => 32,
//    '#disabled' => $type->locked,
    '#machine_name' => array('exists' => 'xoxo_entity_exists'),
    '#description' => t('A unique machine-readable type for this entity.'),
    '#default_value' => $itemInfo ? $itemInfo['type'] : $itemInfo,
  );
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t("Administrative description"),
    '#rows' => 2,
    '#description' => t("Object english extended description for administration screens and help. This must be plain text."),
    '#default_value' => $itemInfo ? $itemInfo['description'] : NULL,
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => $itemInfo ? t("Update") : t("Create"),
    '#submit' => array('xoxo_entity_custom_edit_bundle_form_submit'),
  );
  if ($itemInfo) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t("Delete"),
      '#submit' => array('xoxo_entity_custom_edit_bundle_form_submit_delete')
    );
  }
  return $form;
}

/**
 * Submit bundle.
 */
function xoxo_entity_custom_edit_bundle_form_submit($form, &$form_state) {
  require_once dirname(__FILE__) . '/xoxo_entity.helpers.inc';

  $variableName = 'xoxo_entities_items_' .  $form['#registry_type'];
  $itemType = $form_state['values']['entityType'];

  // FIXME: Change this ugly variable.
  $items = variable_get($variableName, array());

  $items[$itemType] = array(
    'name' => $form_state['values']['name'],
    'class' => 'Xoxo_Object',
    'description' => $form_state['values']['description'],
  );

  // Set the variable only once we created the table.
  variable_set($variableName, $items);
  drupal_flush_all_caches();

  $form_state['redirect'] = 'admin/objects/objects/' . $form['#registry_type'];
}

/**
 * Theme function for registries overview form.
 */
function theme_xoxo_entity_overview(&$variables) {
  $form = &$variables['form'];
  drupal_add_css(drupal_get_path('module', 'oox') . '/oox.admin.css');

  // Prepare table rendering.
  $headers = array(t("Name"), t("Type"), t("Description"), t("Bundles"), t("Custom"), t("Module"), t("Operations"));
  $rows = array();

  foreach (element_children($form['entities']) as $key) {
    $row = array();
    $row['data'][] = drupal_render($form['entities'][$key]['name']);
    $row['data'][] = drupal_render($form['entities'][$key]['type']);
    $row['data'][] = drupal_render($form['entities'][$key]['description']);
    $row['data'][] = drupal_render($form['entities'][$key]['item_count']);
    $row['data'][] = drupal_render($form['entities'][$key]['custom']);
    $row['data'][] = drupal_render($form['entities'][$key]['module']);
    $row['data'][] = drupal_render($form['entities'][$key]['operations']);
    $rows[] = $row;
  }

  return theme('table', array(
    'header' => $headers,
    'rows' => $rows,
    'attributes' => array('class' => array('oox-table')),
  )) . drupal_render_children($form);
}
